  
  


; sample code


; MC56F802x3x PLL defines

CLKGEN      EQU $F130
CTRL        EQU $F130 ; CLKGEN + 0  ; Control 
DIVBY       EQU $F131 ; CLKGEN + 1  ; Divide By
STAT        EQU $F132 ; CLKGEN + 2  ; Status
OCTRL       EQU $F135 ; CLKGEN + 5  ; Oscillator Control
CLKCHK      EQU $F136 ; CLKGEN + 6  ; Clock Check 
PROT        EQU $F137 ; CLKGEN + 7  ; Protection


; flash registers  

FM_BASE     EQU $00F400 
FMCLKD      EQU $00F400 ; FM_BASE + 0   clock divider register

FMOPT1      EQU $00F41B ; FM_BASE + 1B  Reserved for trim cap setting 
                        ; of the relaxation oscillator 
                        ; (on chips other than 56835/836/837/838)
                                               
fmClkDiv    EQU 40      ; for system clock 32 MHZ



; OMR mode bits

NL_MODE			EQU		$8000
CM_MODE			EQU		$0100
XP_MODE	 		EQU		$0080
R_MODE	 		EQU		$0020
SA_MODE	 		EQU		$0010





; init

	section startup

	XREF	F_stack_addr
	
	
	
	
	org	p:

	
	GLOBAL Finit_MC56F803x_

	SUBROUTINE "Finit_MC56F803x_",Finit_MC56F803x_,Finit_MC56F803x_END-Finit_MC56F803x_

Finit_MC56F803x_:

	nop
	
	
	
; Alternative Download Sequence (ALS)
; when target_code_sets_hfmclkd 1 in flash cfg (default is off)
; if ALS is used, set no breakpoints before next instruction

; reset flash Clock Divider Register (FMCLKD) relative to PLL freq
; see Peripheral Reference Manual for value calculation
; this ensures flash breakpoints are set 
; with proper flash freq for any PLL freq

	move.w #fmClkDiv,x:>>FMCLKD

; power down the crystal osciallator circuit
; by settingthe octrl's CLK_MODE
	bfset	#$1000,x:>>OCTRL

	
	
; get trim value and set trim reg
	
	move.l   #OCTRL,R1    ; set R1 and R0 to OCTRL
	move.l   #OCTRL,R0
	move.w   X:(R0),A     ; set A to current OSCT value
	move.w   #-1024,B     ; set B to mask
	and.w    A,B          ; mask A
	move.l   #FMOPT1,R0   ; set R0 to factory-flashed trim value
	move.w   X:(R0),A     ; set A
	or.w     B,A          ; or A and B to get full OCTRL value
	move.w   A1,X:(R1)    ; set OCTRL
	


	 	    
; setup the PLL (phase locked loop)
								

    bfset #$0080,x:>>CTRL          ; set lock detector on 
    bfclr #$0010,x:>>CTRL          ; power on PLL 

    nop 

wait_for_lock: 
    bftsth #$0020,x:>>STAT         ; wait for lock 
    jcc wait_for_lock                     
    nop 
                
    move.w #$0082,x:>>CTRL         ; switch system clock to PLL postscalar                   


	
	

; setup the OMR with the values required by C

	bfset	#NL_MODE,omr    ; ensure NL=1  (enables nested DO loops)
	nop
	nop
	
	; ensure CM=0  (optional for C)
    ; ensure XP=0 to enable harvard architecture
    ; ensure R=0  (required for C)
    ; ensure SA=0 (required for C)
    
	bfclr	#(CM_MODE|XP_MODE|R_MODE|SA_MODE),omr		




; setup the m01 register for linear addressing

	move.w	#-1,x0
	moveu.w	x0,m01      ; set the m register to linear addressing
				    
	moveu.w	hws,la      ; clear the hardware stack
	moveu.w	hws,la
	nop
	nop





CALLMAIN:                           ; initialize compiler environment

                                    ; initialize the stack
	move.l #>>F_Lstack_addr,r0
	bftsth #$0001,r0
	bcc noinc
	adda #1,r0
noinc:
	tfra	r0,sp				    ; set stack pointer too
	move.w	#0,r1
	nop
	move.w	r1,x:(sp)               ; put zero at top of stack
	adda	#1,sp	                ; increment stack
	move.w	r1,x:(sp)               ; put another zero
	
	

; utilities

	
	
 	jsr		F__zeroBSS        		; fill BSS space with zeroes
	jsr     F__romCopy              ; do any rom-to-ram copy
	                                ; LCF sets rom-to-ram flag 
	
    nop


; call main()

	move.w	#0,y0                   ; pass parameters to main()
	move.w	#0,R2
	move.w	#0,R3

	jsr	 	Fmain                   ; call the users program



                                    ; end of program
                                    
                                    ; hostIO cleanup with exit_halt
    jsr     Fexit_halt              ; stationery examples use standard i/o 
                                    ; such as printf implemented with hostIO 
                                    ; so we use exit_halt here 
                                    ; see runtime file exit_dsp.asm
                                    ; comment out exit_halt if not using hostIO
                                                                     
    
    debughlt                        ; or simple end of program; halt CPU	
	rts 
Finit_MC56F803x_END:

	endsec
	
	
	
	
	