

 

/*  metrowerks sample code  */



/*
	this source illustrates code called by ISR 
	and #pragma directives necessary for ISR-safe code		
*/



#include <stdio.h>
#include "MC56F836x.h"
#include "led.h"
#include "isr.h"

static void delay ();

int led_position = 0;
int led_group = 0;



void initIsr ()
{
	asm (bfclr #$0300,sr);		/* allow lowest priority */
	asm (bfset #$C000,X:IPR7);	/* bits 14 and 15: TMRA_0 IPL interrupt for timer A = priority 2 */
	asm (bfset #$000F,X:IPR2);	/* bits 0-3: IRQA/IRQB = priority 2 */

	/* stop timer 0*/
	asm	(move.w	#$0000,X:TMR_A0_CTRL);
	
	/* timer A channel 0 */
	asm (move.w #$FFFF,X:TMR_A0_CMP1);
	asm (move.w	#$0000,X:TMR_A0_CMP2);
	asm (move.w #$FFFF,X:TMR_A0_LOAD);
	asm (move.w	#$FFFF,X:TMR_A0_CNTR);
	
	/* enable timer0 overflow interrupt */
	asm (move.w #$4000,X:TMR_A0_SCR);
	
	
	/* 
	   set timer0 primary count source (PCS) 
	   for top clock rate from PLL prescaler 
	   
	   IPBus clock divide by 128
	   PCS bits 12-9 -> 1111
	   
	   0x3C30 == 0011110000110000 
	   
	   see MC56F8300UM.pdf
	 */
	 
	asm (move.w	#$3C30,X:TMR_A0_CTRL);    
}



/* Interrupt-safe function: 
A function call that conserves all register content
(excluding PC,SP) across a call. Please note that 
Standard C libary is not interrupt-safe */


/* ISR that call non interrupt-safe functions need
#pragma interrupt saveall to:

1. call a runtime routine that save all register (except SP,PC)
2. return with RTI */
#pragma interrupt saveall

void isrIRQA ()
{
	/* debounce key with delay for simplicity, disadvantage:
	1. high latency
	2. no repoll of switch state after deounce delay */
	
	delay ();
	
	puts("IRQA detected");	                    /* Standard C library call */
	offLED (led_position + 1 + led_group * 3);	/* switch off LED from last group */
	led_group = 0;
}





/* ISR that call non interrupt-safe functions need
#pragma interrupt saveall to:

1. call a runtime routine that save all register (except SP,PC)
2. return with RTI */

#pragma interrupt saveall

void isrIRQB ()
{
	/* debounce key with delay for simplicity, disadvantage:
	1. high latency
	2. no repoll of switch state after deounce delay */
	
	delay ();
	
	puts("IRQB detected");	                    /* Standard C library call */
	offLED (led_position + 1 + led_group * 3);	/* switch off LED from last group */
	led_group = 1;
}





/* ISR that does not call interrupt-unsafe functions can use #pragma interrupt to:
1. save all used register
2. return with RTI
*/

#pragma interrupt

void isrTimerA0Compare()
{	
	/* next LED */
	offLED (led_position + 1 + led_group * 3);
	led_position++;
	led_position = led_position % 3;
	onLED (led_position + 1 + led_group * 3);
	
	/* clear compare status flags */
	asm (move.w #$4000,X:TMR_A0_SCR);
}





#pragma interrupt called
static void delay ()
{
	long i;
	
	for (i = 0; i < 0x7FFF; i++)
	{
		/* more nop's for consistence across optimization */
		asm (nop);
		asm (nop);
		asm (nop);
		asm (nop);
		asm (nop);
		asm (nop);
		asm (nop);
		asm (nop);
		asm (nop);
		asm (nop);
		asm (nop);
		asm (nop);
		asm (nop);
		asm (nop);
		asm (nop);
		asm (nop);
	}
}

